unit koutex_f;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Menus;

const Velikost = 256;

type
  TForm1 = class(TForm)
    Timer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }

    RC : HGLRC;
    Model : integer;
    buffer : array[1..Velikost,1..Velikost] of record
                                             blue : byte;
                                             green : byte;
                                             red : byte
                                           end;
    procedure Koule;
    procedure InicializaceOpenGL;
    procedure ZobrazitScenu;
    procedure PixelovyFormat(DC : HDC);
    procedure NastaveniProstoru;
    procedure VlastnostiObjektu;
    procedure LoadBitmap(Path : string);
    procedure VymazatPlochu(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure Svetlo1;
  public
    { Public declarations }
    X,Y,Z: real;
    UhelX,UhelY,UhelZ: integer;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses OpenGL;

const PFDError = 'Chyba pi nastaven PFD.';

procedure TForm1.Koule;
var
  Cesta : string;
  qObj: GLUquadricObj;
begin

  Cesta := ExtractFilePath(Application.ExeName);
  glNewList(Model,GL_COMPILE);
  LoadBitmap(Cesta + 'Image1.bmp');
  glTexImage2d(GL_Texture_2D,0,3,Velikost,Velikost,0,GL_RGB,GL_Unsigned_byte,@buffer);

  qobj := gluNewQuadric();                    // vytvo nov tyhelnk
  gluQuadricTexture(qobj, true);              // otexturuje kouli
  gluSphere(qobj, 0.75, 20, 20);              // zobraz kouli s polomrem 0.75 a st 20 na 20 tyhelnk

glEndList;

end;

procedure TForm1.InicializaceOpenGL;
begin
  PixelovyFormat(Canvas.Handle);
  RC := wglCreateContext(Canvas.Handle);
  wglMakeCurrent(Canvas.Handle,RC);
  glClearColor(0.0,0.0,0.0,0.0);
  glShadeModel(GL_SMOOTH);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_Texture_2D);
  glPixelStorei(GL_Unpack_Alignment,2);
  glTexParameterf(GL_Texture_2D,GL_Texture_Wrap_S,GL_Repeat);
  glTexParameterf(GL_Texture_2D,GL_Texture_Wrap_T,GL_Repeat);
  glTexParameterf(GL_Texture_2D,GL_Texture_Mag_Filter,GL_Linear);
  glTexParameterf(GL_Texture_2D,GL_Texture_Min_Filter,GL_Linear);
  glTexEnvf(GL_Texture_Env, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  Svetlo1;
  Koule;
  NastaveniProstoru;
  wglMakeCurrent(0,0)
end;

procedure TForm1.PixelovyFormat(DC : HDC);
var PFD : TPixelFormatDescriptor;
    nPixelFormat : integer;
begin
  FillChar(PFD,SizeOf(PFD),0);
  with PFD do
    begin
       nSize := SizeOf(TPixelFormatDescriptor);
       nVersion := 1;
       dwFlags := PFD_DRAW_TO_WINDOW or
                  PFD_SUPPORT_OPENGL or
                  PFD_DOUBLEBUFFER;
       iPixelType := PFD_TYPE_RGBA;
       cColorBits := 24;
       cDepthBits   := 32;
       iLayerType   := PFD_MAIN_PLANE;
    end;
  nPixelFormat := ChoosePixelFormat(DC,@PFD);
  Assert(nPixelFormat <> 0,PFDError);
  SetPixelFormat(DC,nPixelFormat,@PFD)
end;

procedure TForm1.NastaveniProstoru;
begin
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  gluPerspective(30.0,ClientWidth / ClientHeight,1.0,10.0);
  glViewport(0,0,ClientWidth,ClientHeight);
end;

procedure TForm1.VlastnostiObjektu;
begin
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;
  glTranslatef(X,Y,Z);				//pozice kamery
  glTranslatef(0,0,0);				//sted rotace
  glRotatef(280, 1, 0, 0);			//rotace koule v ose X
  glRotatef(20, 0, 1, 0);			//rotace koule v ose Y
  glRotatef(UhelZ, 0, 0, 1);			//rotace koule v ose Z
  UhelZ := UhelZ + 10;
end;


procedure TForm1.LoadBitmap(Path : string);
//  nate bitmapu o velikosti 256 x 256 pixelu ze souboru
//  prohodi ervenou a modrou sloku
//  bitmapu jin velikosti je nutno upravit tak, aby
//  jeji ka a vka byla mocninou dvou
var
 X : TImage;
 bih : TBitmapInfoHeader;
 bi : TBitmapInfo;
 b : byte;
 i,j : word;
begin
  X := TImage.Create(Self);
  X.Picture.LoadFromFile(Path);
  with bih do
    begin
      biSize := SizeOf(bih);
      biWidth := Velikost;
      biHeight := Velikost;
      biPlanes := 1;
      biBitCount := 24;
      biCompression := BI_RGB;
      biSizeImage := Velikost;
      biXPelsPerMeter := 0;
      biYPelsPerMeter := 0;
      biClrImportant := 0;
      biClrUsed := 0;
    end;
  bi.bmiHeader := bih;
  GetDIBits(X.Picture.Bitmap.Canvas.Handle,X.Picture.Bitmap.Handle,
            0,Velikost,@buffer,bi,dib_rgb_colors);
  for i := 1 to Velikost do
    for j := 1 to Velikost do
      begin
        b := buffer[i,j].red;
        buffer[i,j].red := buffer[i,j].blue;
        buffer[i,j].blue := b;
      end;
  X.Free
end;

procedure TForm1.ZobrazitScenu;
begin
  wglMakeCurrent(Canvas.Handle,RC);
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  VlastnostiObjektu;
  glCallList(Model);
  glFlush;
  SwapBuffers(Canvas.Handle);
  wglMakeCurrent(0,0)
end;

procedure TForm1.VymazatPlochu(var Message: TWMEraseBkgnd);
begin
  Message.Result := 1
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  UhelX := 0;
  UhelY := 0;
  UhelZ := 0;
  X := 0; Y := 0; Z := -4.0;
  Model := 1;
  InicializaceOpenGL
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  wglDeleteContext(RC)
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  ZobrazitScenu
end;

procedure TForm1.TimerTimer(Sender: TObject);
begin
  Repaint;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  wglMakeCurrent(Canvas.Handle,RC);
  NastaveniProstoru;
  ZobrazitScenu;
  wglMakeCurrent(0,0);
end;


procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_UP: begin  //ipka nahoru
        Z := Z + ((-sin((UhelY - 90) * (Pi/180))) * 0.1);
        X := X - ((sin((UhelY) * (Pi/180))) * 0.1);
    end;
  end;
  case Key of
    VK_DOWN: begin //ipka dolu
      Z := Z - ((-sin((UhelY - 90) * (Pi/180))) * 0.1);
      X := X + ((sin((UhelY) * (Pi/180))) * 0.1);
    end;
  end;
end;

procedure TForm1.Svetlo1;
const
  glfLightAmbient : Array[0..3] of GLfloat = (0.1, 0.1, 0.1, 1.0);  //GL_AMBIENT = ambientn svtlo, svtlo kter z i kdy jsou svtla vypnuta
  glfLightDiffuse : Array[0..3] of GLfloat = (1.0, 1.0, 1.0, 1.0);  //GL_DIFFUSE = barva svtla, kter zdroj vyzauje do okol
  glfLightSpecular: Array[0..3] of GLfloat = (0.5, 0.5, 0.5, 1.0);  //GL_SPECULAR = zrcadlov odraz svtla
  glfLightShininess: GLFloat = 127;                                 //GL_SHININESS = intenzita svtelnho odlesku
  glflightPosition: Array[0..3] of GLfloat = (-10.0, 0.0, 10.0, 1.0);   //GL_Position = pozice svtelnho bodu v prostoru
  glfSpotDirection: Array[0..3] of GLfloat = ( 10.0, 0.0, -10.0, 1.0);    // GL_SPOT_DIRECTION = zadn smru svtelnho kuele
  glfLightEmission: Array[0..3] of GLfloat = (0.0,0.0,0.0,1.0);     // svtlo, kter vyzauje urit objekt, nepsob jako zdroj svtla
  glfLinearAttenuation: GLFloat = 0;                                // GL_LINEAR_ATTENUATION = linern slbnut intenzity svtla
  glfSpotExponent: GLFloat = 1;                                     //GL_SPOT_EXPONENT = charakterizuje zpsob rozptylu svtla pi odrazu
  glfSpotCutoff:GLFloat = 180;                                       //GL_SPOT_CUTOFF = hel, nm se kuel svtla rozevr od sv osy



begin
  glEnable(GL_DEPTH_TEST);

  glLightfv(GL_LIGHT1, GL_AMBIENT, @glfLightAmbient);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, @glfLightDiffuse);
  glLightfv(GL_LIGHT1, GL_SPECULAR, @glfLightSpecular);
  glLightfv(GL_LIGHT1, GL_SHININESS, @glfLightShininess);
  glLightfv(GL_LIGHT1, GL_POSITION,@glfLightPosition);
  glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, @glfSpotDirection);
  glLightfv(GL_LIGHT1, GL_EMISSION, @glfLightEmission);
  glLightfv(GL_LIGHT1, GL_LINEAR_ATTENUATION, @glfLinearAttenuation);
  glLightfv(GL_LIGHT1, GL_SPOT_EXPONENT, @glfSpotExponent);
  glLightfv(GL_LIGHT1, GL_SPOT_CUTOFF, @glfSpotCutoff);




  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT1);  // prvn svtlo

end;


end.
